@@ -61,6 +61,14 @@ from collections import Mapping, MutableMapping
 from abc import ABCMeta, abstractmethod
 
 class PQDictEntry(object):
+    """
+    Abstract class that defines the heap elements that back a PQDict.
+
+    Subclass to customize the ranking behavior of the priority queue. Since the 
+    heap algorithms of PQDict use the "<" comparator to compare entries, 
+    subclasses must implement __lt__.
+
+    """
     __metaclass__ = ABCMeta
     def __init__(self, dkey, pkey):
         self.dkey = dkey
@@ -70,9 +78,6 @@ class PQDictEntry(object):
     def __lt__(self, other):
         return NotImplemented
 
-    # def set_pkey(self, pkey):
-    #     pass
-
     def __eq__(self, other):
         return self.pkey == other.pkey
 
@@ -80,7 +85,13 @@ class PQDictEntry(object):
         return self.__class__.__name__ + \
             "(%s: %s)" % (repr(self.dkey), self.pkey)
 
+    # pkey = property(get_pkey, set_pkey)
+
 class MinPQDEntry(PQDictEntry):
+    """
+    Defines entries for a PQDict backed by a min-heap.
+
+    """
     __init__ = PQDictEntry.__init__
     __eq__ = PQDictEntry.__eq__
 
@@ -88,6 +99,10 @@ class MinPQDEntry(PQDictEntry):
         return self.pkey < other.pkey
 
 class MaxPQDEntry(PQDictEntry):
+    """
+    Defines entries for a PQDict backed by a max-heap.
+
+    """
     __init__ = PQDictEntry.__init__
     __eq__ = PQDictEntry.__eq__
 
@@ -97,18 +112,18 @@ class MaxPQDEntry(PQDictEntry):
 
 class PQDict(MutableMapping):
     """
-    Maps dictionary keys (keys) to priority keys (values). Maintains an
+    Maps dictionary keys (dkeys) to priority keys (pkeys). Maintains an
     internal heap so that the highest priority item can always be obtained in
     constant time. The mapping is mutable so items may be added, removed and
-    have their priorities updated.
+    have their priorities updated without breaking the heap.
 
     """
     # Implementation details:
-    #   - heap (list): stores (dkey,pkey)-pairs as "entries" (PQDEntry objects).
-    #   - nodefinder (dict): maps each dkey to the position of its entry in the 
+    #   * _heap (list): stores (dkey,pkey)-pairs as "entries" (PQDEntry objects).
+    #   * _posfinder (dict): maps each dkey to the position of its entry in the 
     #     heap
-    #   - the < comparator is used to rank entries
-    __slots__ = ('nodefinder', 'heap', 'create_entry')
+    #   * the < comparator is used to rank entries
+    __slots__ = ('_posfinder', '_heap', 'create_entry')
     create_entry = MinPQDEntry
 
     __eq__ = MutableMapping.__eq__
@@ -120,25 +135,26 @@ class PQDict(MutableMapping):
     clear = MutableMapping.clear
     update = MutableMapping.update
     setdefault = MutableMapping.setdefault
-    #fromkeys
+    #fromkeys not included
 
     def __init__(self, *args, **kwargs):
         """
-        Mimics the standard dict constructor:
+        Same input signature as dict:
             Accepts a sequence/iterator of (dkey, pkey) pairs.
             Accepts named arguments or an unpacked dictionary.
         Also accepts a single mapping object to convert it to a pqdict.
 
-        The default priority ranking for entries is in decreasing pkey value
+        The default priority ordering for entries is in decreasing pkey value
         (i.e., a min-pq: LOWER pkey values have a HIGHER rank). This is typical
-        for a scheduler, where the higher ranked tasks have earlier times.
+        for a scheduler, where more highly-ranked tasks have earlier execution
+        times.
 
         """
         if len(args) > 1:
             raise TypeError
 
-        self.heap = []
-        self.nodefinder = {}
+        self._heap = []
+        self._posfinder = {}
         pos = 0
         if args:
             if isinstance(args[0], Mapping):
@@ -148,16 +164,16 @@ class PQDict(MutableMapping):
             try:
                 for dkey, pkey in seq:
                     entry = self.create_entry(dkey, pkey)
-                    self.heap.append(entry)
-                    self.nodefinder[dkey] = pos
+                    self._heap.append(entry)
+                    self._posfinder[dkey] = pos
                     pos += 1
             except TypeError:
                 raise ValueError
         if kwargs:
             for dkey, pkey in kwargs.items():
                 entry = self.create_entry(dkey, pkey)
-                self.heap.append(entry)
-                self.nodefinder[dkey] = pos
+                self._heap.append(entry)
+                self._posfinder[dkey] = pos
                 pos += 1
         self._heapify()
 
@@ -200,14 +216,14 @@ class PQDict(MutableMapping):
         Return number of items in the PQD.
 
         """
-        return len(self.nodefinder)
+        return len(self._posfinder)
 
     def __contains__(self, dkey):
         """
         Return True if dkey is in the PQD else return False.
 
         """
-        return dkey in self.nodefinder
+        return dkey in self._posfinder
 
     def __iter__(self):
         """
@@ -216,7 +232,7 @@ class PQDict(MutableMapping):
         keys sorted by priority.
 
         """
-        for entry in self.heap:
+        for entry in self._heap:
             yield entry.dkey
 
     def __getitem__(self, dkey):
@@ -224,35 +240,35 @@ class PQDict(MutableMapping):
         Return the priority of dkey. Raises a KeyError if not in the PQD.
 
         """
-        return self.heap[self.nodefinder[dkey]].pkey #raises KeyError
+        return self._heap[self._posfinder[dkey]].pkey #raises KeyError
 
     def __setitem__(self, dkey, pkey):
         """
         Assign priority to dictionary key.
 
         """
-        heap = self.heap
-        finder = self.nodefinder
+        heap = self._heap
+        finder = self._posfinder
         try:
             pos = finder[dkey]
         except KeyError:
             # add new entry
-            n = len(self.heap)
-            self.heap.append(self.create_entry(dkey, pkey))
-            self.nodefinder[dkey] = n
+            n = len(self._heap)
+            self._heap.append(self.create_entry(dkey, pkey))
+            self._posfinder[dkey] = n
             self._swim(n)
         else:
             # update existing entry
             heap[pos].pkey = pkey
             parent_pos = (pos - 1) >> 1
             child_pos = 2*pos + 1
-            if parent_pos > 0 and heap[pos] < heap[parent_pos]:
+            if parent_pos > -1 and heap[pos] < heap[parent_pos]:
                 self._swim(pos)
             elif child_pos < len(heap):
-                right_pos = child_pos + 1
-                if (right_pos < len(heap) 
-                        and not heap[child_pos] < heap[right_pos]):
-                    child_pos = right_pos
+                child2_pos = child_pos + 1
+                if (child2_pos < len(heap) 
+                        and not heap[child_pos] < heap[child2_pos]):
+                    child_pos = child2_pos
                 if heap[child_pos] < heap[pos]:
                     self._sink(pos)
 
@@ -261,8 +277,8 @@ class PQDict(MutableMapping):
         Remove item. Raises a KeyError if dkey is not in the PQD.
 
         """
-        heap = self.heap
-        finder = self.nodefinder
+        heap = self._heap
+        finder = self._posfinder
 
         # Remove very last item and place in vacant spot. Let the new item
         # sink until it reaches its new resting place.
@@ -271,23 +287,24 @@ class PQDict(MutableMapping):
         except KeyError:
             raise
         else:
-            entry = heap[pos]
-            last = heap.pop(-1)
-            if entry is not last:
-                heap[pos] = last
-                finder[last.dkey] = pos
+            entry_to_delete = heap[pos]
+            last_entry = heap.pop(-1)
+            if entry_to_delete is not last_entry:
+                heap[pos] = last_entry
+                finder[last_entry.dkey] = pos
+
                 parent_pos = (pos - 1) >> 1
                 child_pos = 2*pos + 1
-                if parent_pos > 0 and heap[pos] < heap[parent_pos]:
+                if parent_pos > -1 and heap[pos] < heap[parent_pos]:
                     self._swim(pos)
                 elif child_pos < len(heap):
-                    right_pos = child_pos + 1
-                    if (right_pos < len(heap) 
-                            and not heap[child_pos] < heap[right_pos]):
-                        child_pos = right_pos
+                    child2_pos = child_pos + 1
+                    if (child2_pos < len(heap) and
+                            not heap[child_pos] < heap[child2_pos]):
+                        child_pos = child2_pos
                     if heap[child_pos] < heap[pos]:
                         self._sink(pos)
-            del entry
+            del entry_to_delete
 
     def __copy__(self):
         """
@@ -297,21 +314,22 @@ class PQDict(MutableMapping):
         """
         # We want the two PQDs to behave as different schedules on the same
         # set of dkeys. As a result:
-        #   - The new heap list contains copies of all entries because PQDEntry
-        #     objects are mutable and should not be shared by two PQDicts.
-        #   - The new nodefinder dict (dkey->heap positions) must be a copy of 
-        #     the old nodefinder dict since it maps the same dkeys to positions 
+        #   - The new heap list contains copies of all entries because 
+        #     PQDictEntry objects are mutable and should not be shared by two 
+        #     PQDicts.
+        #   - The new _posfinder dict (dkey->heap positions) must be a copy of 
+        #     the old _posfinder dict since it maps the same dkeys to positions 
         #     in a different list.
         from copy import copy
         other = self.__class__()
-        other.heap = [copy(entry) for entry in self.heap]
-        other.nodefinder = copy(self.nodefinder)
+        other._heap = [copy(entry) for entry in self._heap]
+        other._posfinder = copy(self._posfinder)
         return other
     copy = __copy__
 
     def __repr__(self):
         things = ', '.join(['%s: %s' % (repr(entry.dkey), entry.pkey) 
-                                for entry in self.heap])
+                                for entry in self._heap])
         return self.__class__.__name__ + '({' + things  + '})'
 
     __marker = object()
@@ -322,8 +340,8 @@ class PQDict(MutableMapping):
         KeyError is raised.
 
         """
-        heap = self.heap
-        finder = self.nodefinder
+        heap = self._heap
+        finder = self._posfinder
 
         try:
             pos = finder.pop(dkey)
@@ -332,24 +350,25 @@ class PQDict(MutableMapping):
                 raise
             return default
         else:
-            delentry = heap[pos]
-            last = heap.pop(-1)
-            if delentry is not last:
-                heap[pos] = last
-                finder[last.dkey] = pos
+            entry_to_delete = heap[pos]
+            pkey = entry_to_delete.pkey
+            last_entry = heap.pop(-1)
+            if entry_to_delete is not last_entry:
+                heap[pos] = last_entry
+                finder[last_entry.dkey] = pos
+
                 parent_pos = (pos - 1) >> 1
                 child_pos = 2*pos + 1
-                if parent_pos > 0 and heap[pos] < heap[parent_pos]:
+                if parent_pos > -1 and heap[pos] < heap[parent_pos]:
                     self._swim(pos)
                 elif child_pos < len(heap):
-                    right_pos = child_pos + 1
-                    if (right_pos < len(heap) 
-                            and not heap[child_pos] < heap[right_pos]):
-                        child_pos = right_pos
+                    child2_pos = child_pos + 1
+                    if (child2_pos < len(heap) 
+                            and not heap[child_pos] < heap[child2_pos]):
+                        child_pos = child2_pos
                     if heap[child_pos] < heap[pos]:
                         self._sink(pos)
-            pkey = delentry.pkey
-            del delentry
+            del entry_to_delete
             return pkey
 
     def popitem(self):
@@ -358,18 +377,18 @@ class PQDict(MutableMapping):
 
         """
         try:
-            last = self.heap.pop(-1)
+            last = self._heap.pop(-1)
         except IndexError:
             raise KeyError
         else:
-            if self.heap:
-                entry = self.heap[0]
-                self.heap[0] = last
-                self.nodefinder[last.dkey] = 0
+            if self._heap:
+                entry = self._heap[0]
+                self._heap[0] = last
+                self._posfinder[last.dkey] = 0
                 self._sink(0)
             else:
                 entry = last
-            self.nodefinder.pop(entry.dkey)
+            self._posfinder.pop(entry.dkey)
             return entry.dkey, entry.pkey
 
     def additem(self, dkey, pkey):
@@ -377,7 +396,7 @@ class PQDict(MutableMapping):
         Add a new item. Raises KeyError if item is already in the PQD.
 
         """
-        if dkey in self.nodefinder:
+        if dkey in self._posfinder:
             raise KeyError
         self[dkey] = pkey
 
@@ -387,7 +406,7 @@ class PQDict(MutableMapping):
         not in the PQD.
 
         """
-        if dkey not in self.nodefinder:
+        if dkey not in self._posfinder:
             raise KeyError
         self[dkey] = new_pkey
 
@@ -397,7 +416,7 @@ class PQDict(MutableMapping):
 
         """
         try:
-            entry = self.heap[0]
+            entry = self._heap[0]
         except IndexError:
             raise KeyError
         return entry.dkey, entry.pkey
@@ -437,19 +456,20 @@ class PQDict(MutableMapping):
             return
 
     def _heapify(self):
-        n = len(self.heap)
+        n = len(self._heap)
         for pos in reversed(range(n//2)):
             self._sink(pos)
 
     def _sink(self, top=0):
-        heap = self.heap
-        finder = self.nodefinder
+        # Floyd's "sink-to-the-bottom-then-swim" algorithm (1964)
+        heap = self._heap
+        finder = self._posfinder
 
-        # Peel off top item
+        # Grab the top item
         pos = top
         entry = heap[pos]
 
-        # Sift up a trail of child nodes
+        # Yank up a chain of child nodes
         child_pos = 2*pos + 1
         while child_pos < len(heap):
             # Choose the index of smaller child.
@@ -457,7 +477,7 @@ class PQDict(MutableMapping):
             if right_pos < len(heap) and not heap[child_pos] < heap[right_pos]:
                 child_pos = right_pos
 
-            # Move the smaller child up.
+            # Move it up one level.
             child_entry = heap[child_pos]
             heap[pos] = child_entry
             finder[child_entry.dkey] = pos
@@ -465,20 +485,20 @@ class PQDict(MutableMapping):
             pos = child_pos
             child_pos = 2*pos + 1
 
-        # We are now at a leaf. Put item there and let it swim until it reaches
-        # its new resting place.
+        # We are left with a "vacant" leaf. Put our item there and let it swim 
+        # until it reaches its new resting place.
         heap[pos] = entry
         finder[entry.dkey] = pos
         self._swim(pos, top)
 
     def _swim(self, pos, top=0):
-        heap = self.heap
-        finder = self.nodefinder
+        heap = self._heap
+        finder = self._posfinder
 
-        # Remove item from its place
+        # Grab the item from its place
         entry = heap[pos]
 
-        # Bubble item up by sifting parents down until finding a place it fits.
+        # Pull parents down until we find a place where the item fits.
         while pos > top:
             parent_pos = (pos - 1) >> 1
             parent_entry = heap[parent_pos]
